##############################################
# R script for illustrating the analysis of
# bipartite network dynamics with RSiena.
# version of 24 June 2021
# written by Christian Steglich
##############################################

# load RSiena commands:
library(RSiena)
packageVersion("RSiena")

# set working directory to where the data are:
setwd("<put your directory name here>")
list.files()

# read the datasets:
load("Glasgow-friendship.RData")
load("Glasgow-lifestyle.RData")
load("Glasgow-demographic.RData")
ls()

# recode valued friendship data: 
friendship.1[friendship.1==2] <- 1
friendship.2[friendship.2==2] <- 1
friendship.3[friendship.3==2] <- 1

# find out number of nodes in two nodesets:
nrpupils <- nrow(friendship.1)
nrmusicItems <- ncol(music1)

# classify music items:
rock <- 1*(colnames(music1)%in%c("rock","heavy","indie","grunge"))
dance <- 1*(colnames(music1)%in%c("chart","dance","techno","rave"))
urban <- 1*(colnames(music1)%in%c("reggae","house","rap","hiphop"))
elite <- 1*(colnames(music1)%in%c("folk","jazz","classical"))
sixty <- 1*(colnames(music1)=="sixty")

# define different node sets:
pupils <- sienaNodeSet(nrpupils,nodeSetName="pupils")
genres <- sienaNodeSet(nrmusicItems,nodeSetName="genres")

# THE ANALYSIS THAT FOLLOWS IS LIMITED TO THE PERIOD
# T1->T2 FOR REASONS OF COMPUTATION TIME ONLY!

# identify dependent variables for the analysis:
music <- sienaDependent(array(c(music1,music2),
	dim=c(nrpupils,nrmusicItems,2)),type="bipartite",
	nodeSet=c("pupils","genres"))
friendship <- sienaDependent(array(c(friendship.1,friendship.2),
	dim=c(nrpupils,nrpupils,2)),type="oneMode",
	nodeSet="pupils")

# include exogenous predictor variables:
sexF <- coCovar(sex.F,nodeSet="pupils")  
rock <- coCovar(rock,nodeSet="genres")  
dance <- coCovar(dance,nodeSet="genres")  
urban <- coCovar(urban,nodeSet="genres")  
elite <- coCovar(elite,nodeSet="genres")  
sixty <- coCovar(sixty,nodeSet="genres")  

# combine data for the analysis:
bipData <- sienaDataCreate(friendship,music,sexF,
	rock,dance,urban,elite,sixty,
	nodeSets=list(pupils,genres))

# generate initial descriptive outputfile:
print01Report(bipData,modelname="Bipartite-illustration")
# Take a look at the generated output file "Bipartite-illustration.out"
# to see how RSiena interpreted the directives provided.

# get effects table for model specification:
bipEffects <- getEffects(bipData)
effectsDocumentation(bipEffects)

# specify the 'friendship' part of the model:
bipEffects <- includeEffects(bipEffects,transTrip,transRecTrip,
	name="friendship")
	# "Is there transitive closure of the friendship network?"
	# "Is reciprocity in triads weaker than in isolated dyads?"
bipEffects <- includeEffects(bipEffects,sameX,
	name="friendship",interaction1="sexF")
	# "Is friendship sex-segregated?"
bipEffects <- includeEffects(bipEffects,from,
	name="friendship",interaction1="music")
	# "Do shared music genres imply friendship?"

# add effects in the 'music' part of the model:
bipEffects <- includeEffects(bipEffects,cycle4,
	name="music")
	# "Does music listening obey the 'amazon recommender' effect?"
bipEffects <- includeEffects(bipEffects,to,
	name="music",interaction1="friendship")
	# "Are friends' music genres adopted?"
bipEffects <- includeEffects(bipEffects,egoX,
	name="music",interaction1="sexF")
	# "Do girls listen to more genres than boys?"
bipEffects <- includeEffects(bipEffects,altX,
	name="music",interaction1="rock")
	# "Is rock more popular than the reference style (dance)?"
bipEffects <- includeEffects(bipEffects,altX,
	name="music",interaction1="urban")
	# "Is urban more popular than the reference style (dance)?"
bipEffects <- includeEffects(bipEffects,altX,
	name="music",interaction1="elite")
	# "Is elite more popular than the reference style (dance)?"
	
# How does model specification look like?
bipEffects

# create estimation options object:
bipOptions <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='bipartite-results')

# estimate (took ~4.5 minutes on my machine - skip in class!):
bipResults <- siena07(bipOptions,data=bipData,effects=bipEffects,
	batch=FALSE,verbose=FALSE,useCluster=TRUE,initC=TRUE,nbrNodes=2)

# save intermediate results:
#save(bipResults,file="bipResults.RData")
load("bipResults.RData")
bipResults

# CONCLUSIONS: Next to known results from other studies
# (parameters 1-6, 8-9) we find the following:
# - no evidence for a mechanism buy which shared music listening
#   habits would lead to friendship (parameter 7)
# - strong evidence for the 'Amazon recommender' 4-cycle effect
#   (parameter 10, actors listen to genres that others who
#   listen to the same genre also listen to - obviously a
#   clustering phenomenon that does not require Amazon.com)
# - no evidence for girls differing from boys in the amount of
#   genres they listen to (parameter 11)
# - "rock", "urban" and "elite" styles all are less popular than
#   "dance", the latter two significantly so (parameters 12-14)
# - strong evidence for actors copying their friends' listening
#   habits (bipartite influence, parameter 15)
